@@ -12,13 +12,17 @@ import cwltool.context
 def cli():
    pass
 
+# Start of the docker-compose using d2s-cwl-workflows yaml
+docker_compose_cmd = 'docker-compose -f d2s-cwl-workflows/docker-compose.yaml '
+
 # Used for autocompletion
 def get_services_list(ctx, args, incomplete):
     # TODO: automate by parsing the docker-compose.yaml
     return filter(lambda x: x.startswith(incomplete), [ 'demo',
     'graphdb', 'virtuoso', 'tmp-virtuoso', 'blazegraph', 'allegrograph', 'anzograph',
     'into-the-graph', 'ldf-server', 'comunica', 'notebook',
-    'api', 'drill', 'postgres', 'proxy', 'filebrowser', 'rmljob', 'rmltask' ])
+    'api', 'drill', 'postgres', 'proxy', 'filebrowser', 'rmlstreamer', 'rmltask',
+    'neo4j' ])
 def get_datasets_list(ctx, args, incomplete):
     return filter(lambda x: x.startswith(incomplete), os.listdir("./datasets"))
 def get_workflows_list(ctx, args, incomplete):
@@ -71,8 +75,10 @@ def init(ctx, projectname):
     os.system('chmod -R 777 workspace/graphdb-import')
     os.system('mkdir -p workspace/workflow-history')
     os.system('mkdir -p workspace/download/releases/1')
-    click.echo(click.style('[d2s]', bold=True) + ' Downloading RMLStreamer.jar... [80M]')
-    urllib.request.urlretrieve ("https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar", "workspace/RMLStreamer.jar")
+    os.system('cp ~/RMLStreamer.jar workspace/RMLStreamer.jar')
+    if not os.path.exists('/home/vemonet/RMLStreamer.jar'):
+        click.echo(click.style('[d2s]', bold=True) + ' Downloading RMLStreamer.jar... [80M]')
+        urllib.request.urlretrieve ("https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar", "workspace/RMLStreamer.jar")
     # os.system('wget -a workspace/RMLStreamer.jar https://github.com/vemonet/RMLStreamer/raw/fix-mainclass/target/RMLStreamer-1.2.2.jar')
     os.system('chmod +x workspace/RMLStreamer.jar')
 
@@ -109,8 +115,8 @@ def init(ctx, projectname):
 @cli.command()
 def update():
     """Update Docker images"""
-    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml pull')
-    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml build graphdb')
+    os.system(docker_compose_cmd + 'pull')
+    os.system(docker_compose_cmd + 'build graphdb')
     click.echo(click.style('[d2s]', bold=True) + ' All images pulled and built.')
     click.echo(click.style('[d2s]', bold=True) + ' You can now start services (e.g. demo services):')
     click.secho('d2s start demo', bold=True)
@@ -148,10 +154,10 @@ def start(services, deploy):
     
     # Run docker-compose:
     if deploy:
-        os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml -f d2s-cwl-workflows/docker-compose.'
+        os.system(docker_compose_cmd + '-f d2s-cwl-workflows/docker-compose.'
         + deploy + '.yaml up -d --force-recreate ' + services_string)
     else:
-        os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml up -d --force-recreate ' + services_string)
+        os.system(docker_compose_cmd + 'up -d --force-recreate ' + services_string)
 
     # Ask user to create the GraphDB test repository
     click.echo(click.style('[d2s] ', bold=True) + services_string + ' started.')
@@ -176,12 +182,12 @@ def start(services, deploy):
 def stop(services, all):
     """Stop services (--all to stop all services)"""
     if all:
-        os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml down')
+        os.system(docker_compose_cmd + 'down')
         os.system('sudo rm -rf workspace/virtuoso')
         click.echo(click.style('[d2s] ', bold=True) + 'All services stopped.')
     else:
         services_string = " ".join(services)
-        os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml stop ' + services_string)
+        os.system(docker_compose_cmd + 'stop ' + services_string)
         click.echo(click.style('[d2s] ', bold=True) + services_string + ' stopped.')
 
 
@@ -242,20 +248,40 @@ def download(datasets):
 
 @cli.command()
 @click.argument('dataset', autocompletion=get_datasets_list)
+@click.option(
+    '--detached/--watch', default=True, 
+    help='Run in detached mode or watch workflow')
 # @click.option(
 #     '-p', '--parallelism', default='8', # Not working properly.
 #     help='Run in parallel, depends on Task Slots availables')
-def rml(dataset):
+def rml(dataset, detached):
     """Run RML Streamer"""
-    click.echo(click.style('[d2s]', bold=True) + ' Execute mappings from ' 
-        + click.style('datasets/' + dataset + '/mapping/rml-mappings.ttl', bold=True))
-    rmlstreamer_cmd = 'docker exec -d d2s-cwl-workflows_rmljob_1 /opt/flink/bin/flink run /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/graphdb-import/rml-output-' + dataset + '.nt'
+    if (detached):
+        detached_arg = "-d"
+    else:
+        detached_arg = "-it"
+
+    for file in os.listdir('./datasets/' + dataset + '/mapping'):
+     mapping_filename = os.fsdecode(file)
+     if mapping_filename.endswith(".rml.ttl"): 
+        # print(os.path.join(directory, filename))
+        mapping_filepath = 'datasets/' + dataset + '/mapping/' + mapping_filename
+        click.echo(click.style('[d2s]', bold=True) + ' Execute mappings from ' 
+            + click.style(mapping_filepath, bold=True))
+    
+        rmlstreamer_cmd = 'docker exec ' + detached_arg + ' d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -c io.rml.framework.Main /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/' + mapping_filename + ' --outputPath /mnt/workspace/graphdb-import/' + mapping_filename.replace('.', '_') + '-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + mapping_filename + ' - ' + dataset + '"'
+    
+        os.system(rmlstreamer_cmd)
+        click.echo(click.style('[d2s]', bold=True) + ' Output file in ' )
+        click.secho('workspace/graphdb-import/'+ mapping_filename.replace('.', '_') + '-' + dataset + '.nt', bold=True)
+    
+    click.echo(click.style('[d2s]', bold=True) + ' Check the jobs running at ' 
+            + click.style('http://localhost:8078/#/job/running', bold=True))
+    # Try parallelism:
+    # rmlstreamer_cmd = 'docker exec -d d2s-cwl-workflows_rmlstreamer_1 /opt/flink/bin/flink run -p 4 /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/graphdb-import/rml-output-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + dataset + '" --enable-local-parallel'
+    # Try to use docker-compose, but exec dont resolve from the -f file
+    # rmlstreamer_cmd = docker_compose_cmd + 'exec -d rmlstreamer /opt/flink/bin/flink run /mnt/workspace/RMLStreamer.jar --path /mnt/datasets/' + dataset + '/mapping/rml-mappings.ttl --outputPath /mnt/workspace/graphdb-import/rml-output-' + dataset + '.nt --job-name "[d2s] RMLStreamer ' + dataset + '"'
     # print(rmlstreamer_cmd)
-    os.system(rmlstreamer_cmd)
-    click.echo(click.style('[d2s]', bold=True) + ' Check the job running at ' 
-        + click.style('http://localhost:8078/#/job/running', bold=True))
-    click.echo(click.style('[d2s]', bold=True) + ' Output file in ' 
-        + click.style("workspace/graphdb-import/rml-output-" + dataset + ".nt", bold=True))
 
 @cli.command()
 @click.argument('workflow', autocompletion=get_workflows_list)
@@ -279,8 +305,8 @@ def run(workflow, dataset, get_mappings, detached):
     click.echo(click.style('[d2s] ', bold=True) 
         + 'Restart tmp Virtuoso and delete file in '
         + click.style('workspace/output', bold=True))
-    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml stop tmp-virtuoso')
-    os.system('docker-compose -f d2s-cwl-workflows/docker-compose.yaml up -d --force-recreate tmp-virtuoso')
+    os.system(docker_compose_cmd + 'stop tmp-virtuoso')
+    os.system(docker_compose_cmd + 'up -d --force-recreate tmp-virtuoso')
     # TODO: fix this dirty Virtuoso deployment 
     # Virtuoso unable to handle successive bulk load + permission issues + load.sh in the virtuoso containers
     # I don't know how, they managed to not put it in the container... They had one job...
